@@ -29,6 +29,7 @@
 
 #include "entities.h"
 #include "global.h"
+#include "clientcapabilities.h"
 
 namespace Akonadi {
     class Handler;
@@ -72,13 +73,19 @@ public:
     void setSessionId( const QByteArray &id );
     QByteArray sessionId() const;
 
+    const ClientCapabilities& capabilities() const;
+    void setCapabilities(const ClientCapabilities &capabilities);
+
+    /** Returns @c true if permanent cache verification is enabled. */
+    bool verifyCacheOnRetrieval() const;
+
 protected Q_SLOTS:
     void slotDisconnected();
     /**
      * New data arrived from the client. Creates a handler for it and passes the data to the handler.
      */
     void slotNewData();
-    void slotResponseAvailable( const Response& );
+    void slotResponseAvailable( const Akonadi::Response& response );
     void slotConnectionStateChange( ConnectionState );
 
 protected:
@@ -104,6 +111,8 @@ private:
     QByteArray m_sessionId;
     ImapStreamParser *m_streamParser;
     Resource m_resourceContext;
+    ClientCapabilities m_clientCapabilities;
+    bool m_verifyCacheOnRetrieval;
 };
 
 }
